@@ -118,6 +118,8 @@ int Assemble(struct supertype *st, char *mddev, int mdfd,
 	mddev_dev_t tmpdev;
 	struct mdinfo info;
 	struct mddev_ident_s ident2;
+	char *avail;
+	int nextspare = 0;
 	
 	vers = md_get_version(mdfd);
 	if (vers <= 0) {
@@ -193,7 +195,7 @@ int Assemble(struct supertype *st, char *mddev, int mdfd,
 			super = NULL;
 		}
 		
-		dfd = open(devname, O_RDONLY|O_EXCL, 0);
+		dfd = dev_open(devname, O_RDONLY|O_EXCL);
 		if (dfd < 0) {
 			if ((inargv && verbose >= 0) || verbose > 0)
 				fprintf(stderr, Name ": cannot open device %s: %s\n",
@@ -202,11 +204,9 @@ int Assemble(struct supertype *st, char *mddev, int mdfd,
 			/* Impossible! */
 			fprintf(stderr, Name ": fstat failed for %s: %s\n",
 				devname, strerror(errno));
-			close(dfd);
 		} else if ((stb.st_mode & S_IFMT) != S_IFBLK) {
 			fprintf(stderr, Name ": %s is not a block device.\n",
 				devname);
-			close(dfd);
 		} else if (!tst && (tst = guess_super(dfd)) == NULL) {
 			if ((inargv && verbose >= 0) || verbose > 0)
 				fprintf(stderr, Name ": no recogniseable superblock\n");
@@ -214,13 +214,12 @@ int Assemble(struct supertype *st, char *mddev, int mdfd,
 			if ((inargv && verbose >= 0) || verbose > 0)
 				fprintf( stderr, Name ": no RAID superblock on %s\n",
 					 devname);
-			close(dfd);
 		} else {
 			tst->ss->getinfo_super(&info, &ident2, super);
-			close(dfd);
 		}
+		if (dfd >= 0) close(dfd);
 
-		if (ident->uuid_set &&
+		if (ident->uuid_set && (!update && strcmp(update, "uuid")!= 0) &&
 		    (!super || same_uuid(info.uuid, ident->uuid, tst->ss->swapuuid)==0)) {
 			if ((inargv && verbose >= 0) || verbose > 0)
 				fprintf(stderr, Name ": %s has wrong uuid.\n",
@@ -282,10 +281,24 @@ int Assemble(struct supertype *st, char *mddev, int mdfd,
 			struct stat stb2;
 			fstat(mdfd, &stb2);
 			info.array.md_minor = minor(stb2.st_rdev);
-			
+
+			if (strcmp(update, "uuid")==0 &&
+			    !ident->uuid_set) {
+				int rfd;
+				if ((rfd = open("/dev/urandom", O_RDONLY)) < 0 ||
+				    read(rfd, ident->uuid, 16) != 16) {
+					*(__u32*)(ident->uuid) = random();
+					*(__u32*)(ident->uuid+1) = random();
+					*(__u32*)(ident->uuid+2) = random();
+					*(__u32*)(ident->uuid+3) = random();
+				}
+				if (rfd >= 0) close(rfd);
+				ident->uuid_set = 1;
+			}
+			memcpy(info.uuid, ident->uuid, 16);
 			st->ss->update_super(&info, super, update, devname, verbose);
 			
-			dfd = open(devname, O_RDWR|O_EXCL, 0);
+			dfd = dev_open(devname, O_RDWR|O_EXCL);
 			if (dfd < 0) 
 				fprintf(stderr, Name ": Cannot open %s for superblock update\n",
 					devname);
@@ -319,6 +332,11 @@ int Assemble(struct supertype *st, char *mddev, int mdfd,
 			i = devcnt;
 		else
 			i = devices[devcnt].raid_disk;
+		if (i+1 == 0) {
+			if (nextspare < info.array.raid_disks)
+				nextspare = info.array.raid_disks;
+			i = nextspare++;
+		}
 		if (i < 10000) {
 			if (i >= bestcnt) {
 				unsigned int newbestcnt = i+10;
@@ -359,11 +377,15 @@ int Assemble(struct supertype *st, char *mddev, int mdfd,
 	/* now we have some devices that might be suitable.
 	 * I wonder how many
 	 */
+	avail = malloc(info.array.raid_disks);
+	memset(avail, 0, info.array.raid_disks);
 	okcnt = 0;
 	sparecnt=0;
 	for (i=0; i< bestcnt ;i++) {
 		int j = best[i];
-		int event_margin = !force;
+		int event_margin = 1; /* always allow a difference of '1'
+				       * like the kernel does
+				       */
 		if (j < 0) continue;
 		/* note: we ignore error flags in multipath arrays
 		 * as they don't make sense
@@ -377,13 +399,16 @@ int Assemble(struct supertype *st, char *mddev, int mdfd,
 		if (devices[j].events+event_margin >=
 		    devices[most_recent].events) {
 			devices[j].uptodate = 1;
-			if (i < info.array.raid_disks)
+			if (i < info.array.raid_disks) {
 				okcnt++;
-			else
+				avail[i]=1;
+			} else
 				sparecnt++;
 		}
 	}
-	while (force && !enough(info.array.level, info.array.raid_disks, okcnt)) {
+	while (force && !enough(info.array.level, info.array.raid_disks,
+				info.array.layout,
+				avail, okcnt)) {
 		/* Choose the newest best drive which is
 		 * not up-to-date, update the superblock
 		 * and add it.
@@ -406,7 +431,7 @@ int Assemble(struct supertype *st, char *mddev, int mdfd,
 				devices[chosen_drive].devname, devices[chosen_drive].raid_disk,
 				(int)(devices[chosen_drive].events),
 				(int)(devices[most_recent].events));
-		fd = open(devices[chosen_drive].devname, O_RDWR|O_EXCL);
+		fd = dev_open(devices[chosen_drive].devname, O_RDWR|O_EXCL);
 		if (fd < 0) {
 			fprintf(stderr, Name ": Couldn't open %s for write - not updating\n",
 				devices[chosen_drive].devname);
@@ -434,6 +459,7 @@ int Assemble(struct supertype *st, char *mddev, int mdfd,
 		close(fd);
 		devices[chosen_drive].events = devices[most_recent].events;
 		devices[chosen_drive].uptodate = 1;
+		avail[chosen_drive] = 1;
 		okcnt++;
 		free(super);
 	}
@@ -455,7 +481,7 @@ int Assemble(struct supertype *st, char *mddev, int mdfd,
 		if (!devices[j].uptodate)
 			continue;
 		chosen_drive = j;
-		if ((fd=open(devices[j].devname, O_RDONLY|O_EXCL))< 0) {
+		if ((fd=dev_open(devices[j].devname, O_RDONLY|O_EXCL))< 0) {
 			fprintf(stderr, Name ": Cannot open %s: %s\n",
 				devices[j].devname, strerror(errno));
 			return 1;
@@ -521,7 +547,7 @@ int Assemble(struct supertype *st, char *mddev, int mdfd,
 
 	if (change) {
 		int fd;
-		fd = open(devices[chosen_drive].devname, O_RDWR|O_EXCL);
+		fd = dev_open(devices[chosen_drive].devname, O_RDWR|O_EXCL);
 		if (fd < 0) {
 			fprintf(stderr, Name ": Could open %s for write - cannot Assemble array.\n",
 				devices[chosen_drive].devname);
@@ -599,7 +625,7 @@ int Assemble(struct supertype *st, char *mddev, int mdfd,
 		
 		if (runstop == 1 ||
 		    (runstop == 0 && 
-		     ( enough(info.array.level, info.array.raid_disks, okcnt) &&
+		     ( enough(info.array.level, info.array.raid_disks, info.array.layout, avail, okcnt) &&
 		       (okcnt >= req_cnt || start_partial_ok)
 			     ))) {
 			if (ioctl(mdfd, RUN_ARRAY, NULL)==0) {
@@ -627,7 +653,7 @@ int Assemble(struct supertype *st, char *mddev, int mdfd,
 			fprintf(stderr, Name ": %s assembled from %d drive%s", mddev, okcnt, okcnt==1?"":"s");
 			if (sparecnt)
 				fprintf(stderr, " and %d spare%s", sparecnt, sparecnt==1?"":"s");
-			if (!enough(info.array.level, info.array.raid_disks, okcnt))
+			if (!enough(info.array.level, info.array.raid_disks, info.array.layout, avail, okcnt))
 				fprintf(stderr, " - not enough to start the array.\n");
 			else {
 				if (req_cnt == info.array.raid_disks)
