@@ -51,7 +51,7 @@ int Examine(mddev_dev_t devlist, int brief, int scan, int SparcAdjust, struct su
 	 *   utime, state etc
 	 *
 	 * If (brief) gather devices for same array and just print a mdadm.conf line including devices=
-	 * if devlist==NULL, use conf_get_devs(
+	 * if devlist==NULL, use conf_get_devs()
 	 */
 	int fd; 
 	void *super = NULL;
@@ -62,6 +62,7 @@ int Examine(mddev_dev_t devlist, int brief, int scan, int SparcAdjust, struct su
 		void *super;
 		struct supertype *st;
 		struct mdinfo info;
+		struct mddev_ident_s ident;
 		void *devs;
 		struct array *next;
 		int spares;
@@ -70,7 +71,7 @@ int Examine(mddev_dev_t devlist, int brief, int scan, int SparcAdjust, struct su
 	for (; devlist ; devlist=devlist->next) {
 		struct supertype *st = forcest;
 
-		fd = open(devlist->devname, O_RDONLY);
+		fd = dev_open(devlist->devname, O_RDONLY);
 		if (fd < 0) {
 			if (!scan)
 				fprintf(stderr,Name ": cannot open %s: %s\n",
@@ -91,7 +92,7 @@ int Examine(mddev_dev_t devlist, int brief, int scan, int SparcAdjust, struct su
 		if (err) rv =1;
 
 		if (SparcAdjust)
-			st->ss->update_super(NULL, super, "sparc2.2", devlist->devname, 0);
+			st->ss->update_super(NULL, super, "sparc2.2", devlist->devname,  0);
 		/* Ok, its good enough to try, though the checksum could be wrong */
 		if (brief) {
 			struct array *ap;
@@ -108,9 +109,9 @@ int Examine(mddev_dev_t devlist, int brief, int scan, int SparcAdjust, struct su
 				ap->spares = 0;
 				ap->st = st;
 				arrays = ap;
-				st->ss->getinfo_super(&ap->info, super);
+				st->ss->getinfo_super(&ap->info, &ap->ident, super);
 			} else {
-				st->ss->getinfo_super(&ap->info, super);
+				st->ss->getinfo_super(&ap->info, &ap->ident, super);
 				free(super);
 			}
 			if (!(ap->info.disk.state & MD_DISK_SYNC))
@@ -130,14 +131,17 @@ int Examine(mddev_dev_t devlist, int brief, int scan, int SparcAdjust, struct su
 			char *d;
 			ap->st->ss->brief_examine_super(ap->super);
 			if (ap->spares) printf("   spares=%d", ap->spares);
-			printf("   devices");
-			for (d=dl_next(ap->devs); d!= ap->devs; d=dl_next(d)) {
-				printf("%c%s", sep, d);
-				sep=',';
+			if (brief > 1) {
+				printf("   devices");
+				for (d=dl_next(ap->devs); d!= ap->devs; d=dl_next(d)) {
+					printf("%c%s", sep, d);
+					sep=',';
+				}
 			}
 			free(ap->super);
 			/* FIXME free ap */
-			printf("\n");
+			if (ap->spares || brief > 1)
+				printf("\n");
 		}
 	}
 	return rv;
